/* -*-C-*-

$Id: get-addr.c,v 1.3 2001/06/13 17:31:02 cph Exp $

Copyright (c) 2001 Massachusetts Institute of Technology

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>

int
main (int argc, const char ** argv)
{
  const char * ifname;
  int fd;
  struct ifreq ifr;
  unsigned long addr;

  if (argc == 2)
    ifname = (argv[1]);
  else
    {
      fprintf (stderr, "usage: %s <interface>\n", (argv[0]));
      return (2);
    }

  /* Open a socket.  */
  fd = (socket (AF_INET, SOCK_DGRAM,0));
  if (fd < 0)
    {
      perror ("socket");
      printf ("unknown\n");
      return (0);
    }

  /* Address isn't interesting if the interface is down.  */
  strncpy ((ifr.ifr_name), ifname, IFNAMSIZ);
  if ((ioctl (fd, SIOCGIFFLAGS, (&ifr))) < 0)
    {
      fprintf (stderr, "SIOCGIFFLAGS on %s failed: %s\n", ifname,
	       (strerror (errno)));
      close (fd);
      printf ("unknown\n");
      return (0);
    }
  if (((ifr . ifr_flags) & IFF_UP) == 0)
    {
      close (fd);
      printf ("down\n");
      return (0);
    }

  /* Get the address and print it.  */
  strncpy ((ifr.ifr_name), ifname, IFNAMSIZ);
  if ((ioctl (fd, SIOCGIFADDR, (&ifr))) < 0)
    {
      fprintf (stderr, "SIOCGIFADDR on %s failed: %s\n", ifname,
	       (strerror (errno)));
      close (fd);
      printf ("unknown\n");
      return (0);
    }
  close (fd);
  addr
    = (ntohl ((((struct sockaddr_in *) (& (ifr.ifr_addr))) -> sin_addr)
	      . s_addr));
  printf ("%ld.%ld.%ld.%ld\n",
	  ((addr >> 24) & 0xff),
	  ((addr >> 16) & 0xff),
	  ((addr >> 8) & 0xff),
	  (addr & 0xff));
  return (0);
}
